package jmsltestsuite;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.Frame;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;

import com.softsynth.jmsl.JMSL;
import com.softsynth.jmsl.midi.MidiIOFactory;
import com.softsynth.jmsl.midi.MidiListener;
import com.softsynth.jmsl.midi.MidiParser;
import com.softsynth.jmsl.view.TextAreaOut;

/**
 * Bring up a frame with Midi record/stop/play buttons.
 * 
 * @author Nick Didkovsky 4/09
 */
public class MidiRecorderTest extends Frame implements ActionListener, MidiListener {

	Button recordButton;
	Button stopButton;

	TextArea textArea;

	MidiParser parser;

	final int MODE_RECORDING = 1;
	final int MODE_STOPPED = 0;
	int mode = MODE_STOPPED;

	Checkbox thru;

	public MidiRecorderTest() {
		buildGUI();
		buildRecorder();
		WindowAdapter windowAdapter = new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				JMSL.closeMusicDevices();
				System.exit(0);
			}
		};
		addWindowListener(windowAdapter);
	}

	void buildGUI() {
		removeAll();
		setLayout(new BorderLayout(10, 10));
		textArea = new TextArea(20, 80);
		JMSL.setSTDOut(new TextAreaOut(textArea));
		add("South", textArea);

		Panel p = new Panel();
		p.add(thru = new Checkbox("Thru while recording", true));
		p.add(recordButton = new Button("Record"));
		p.add(stopButton = new Button("Stop"));
		// p.add(playButton = new Button("Play"));
		stopButton.setEnabled(false);
		// playButton.setEnabled(false);
		add("North", p);
		recordButton.addActionListener(this);
		stopButton.addActionListener(this);
		// playButton.addActionListener(this);
	}

	void buildRecorder() {

		parser = new MidiParser();
		parser.addMidiListener(this);
	}

	synchronized void handleStart() {
		JMSL.midi.addMidiParser(parser);

		System.out.println("Start");
		mode = MODE_RECORDING;
		recordButton.setEnabled(false);
		stopButton.setEnabled(true);

	}

	synchronized void handleStop() {
		if (mode == MODE_RECORDING) {
			JMSL.midi.removeMidiParser(parser); // take it out so polling
			// doesn't hog CPU
		}
		mode = MODE_STOPPED;
		recordButton.setEnabled(true);
		stopButton.setEnabled(false);
	}

	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		if (source == recordButton) {
			handleStart();
		} else if (source == stopButton) {
			handleStop();
		}
	}

	void writeMidiFile() {

	}

	public static void main(String args[]) {
//		System.out.println("Midi.typeNoteOn=" + Midi.typeKeyOn);
//		System.exit(0);
		MidiRecorderTest f = new MidiRecorderTest();

		try {
			JMSL.midi = new MidiIOFactory().getMidiIO(MidiIOFactory.MidiIO_JAVASOUND);
			JMSL.midi.openDevices(1, 0);
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		f.pack();
		f.setVisible(true);
		f.addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent e) {
				JMSL.closeMusicDevices();
				System.exit(0);
			}
		});
	}

	public void handleChannelAftertouch(double timeStamp, int channel, int pressure) {
		System.out.println("channel aftertouch");

	}

	public void handleControlChange(double timeStamp, int channel, int id, int value) {
		System.out.println("control");
	}

	public void handleNoteOff(double timeStamp, int channel, int pitch, int velocity) {
		System.out.println("note off");

	}

	public void handleNoteOn(double timeStamp, int channel, int pitch, int velocity) {
		System.out.println("note on");

	}

	public void handlePitchBend(double timeStamp, int channel, int lsb, int msb) {
		System.out.println("pitch bend");

	}

	public void handlePolyphonicAftertouch(double timeStamp, int channel, int pitch, int pressure) {
		System.out.println("poly aftertouch");

	}

	public void handleProgramChange(double timeStamp, int channel, int program) {
		System.out.println("program change");

	}

	public void handleSysEx(double timeStamp, byte[] data) {
		System.out.println("sysex");

	}

}
