package jmsltestsuite;
import java.util.Vector;

import com.softsynth.jmsl.JMSLRandom;
import com.softsynth.jmsl.MusicShape;
import com.softsynth.jmsl.score.*;
import com.softsynth.jmsl.score.transcribe.*;
import com.softsynth.jmsl.util.TimeSignature;

/** 
 * 
 * Transcribe a MusicShapes with wacko tuplets
 * 
 * Dim 0 of MusicShapes must be absolute time.  Use musicShape.integrate(0) to convert durations to abs time
 * 
 * @author Nick Didkovsky, (c) 2002 Nick Didkovsky, All Rights reserved. */

public class TranscribeScore12 {

	public static void main(String args[]) {
		JMSLRandom.randomize();

		// set up allowed subdivisions
		BeatDivisionSchemeList.removeAllBeatDivisionSchemes();

//      binary 16th
        BeatDivisionScheme b = new BeatDivisionScheme();
        b.setTimeSpan(1.0);
        b.setMinTimeSpan(.25);
        b.setMinRequired(1);
        b.setNumberOfDivisions(4);
        b.setConformToBeat(true);
        BeatDivisionSchemeList.addBeatDivisionScheme(b);

//      19 tuplet
        b = new BeatDivisionScheme();
        b.setTimeSpan(4.0);
        b.setMinTimeSpan(4.0);
        b.setMinRequired(4);
        b.setNumberOfDivisions(19);
        b.setConformToBeat(false);
        BeatDivisionSchemeList.addBeatDivisionScheme(b);

		double dur19th= 4.0 / 19;
		MusicShape melody1 = new MusicShape(4);
		for (int i = 0; i < 19; i++) {
			double duration = dur19th;
			double pitch = JMSLRandom.choose(60, 84);
			double amp = JMSLRandom.choose(0.2, 0.7);
			double hold = duration * 0.8;
			melody1.add(duration, pitch, amp, hold);
		}
//		// little test, add some rest events that might have been generates algorithmically
//		melody1.add(.25, 0, 0, 0.9);
//		melody1.add(.25, 0, 0, 0.9);
//		melody1.add(.25, 0, 0, 0.9);
//		melody1.add(.25, 0, 0, 0.9);
//		melody1.add(.75, 0, 0, 0.9);
//		melody1.add(.25, 70, .50, 0.9);
//		melody1.add(1, 60, 0.99, 0.9);
//		melody1.add(1, 72, 0.99, 0.9);
//
//		// add 16ths and 32nds to second MusicShape
//		MusicShape melody2 = new MusicShape(4);
//		double totalDur32 = 0;
//		for (int i = 0; i < 16; i++) {
//			double duration = JMSLRandom.choose() < 0.5 ? 0.25 : 0.125;
//			double pitch = JMSLRandom.choose(48, 60);
//			double amp = JMSLRandom.choose(0.2, 0.7);
//			double hold = duration * 0.8;
//			melody2.add(duration, pitch, amp, hold);
//			totalDur32 += duration;
//		}
//
//		double roundUpDur = totalDur32 - (int) totalDur32;
//		if (roundUpDur != 0) {
//			melody2.add(roundUpDur, 90, 0.5, 0.1);
//		}
//		for (int i = 0; i < 17; i++) {
//			double duration = 1 / 5.0;
//			double pitch = JMSLRandom.choose(48, 60);
//			double amp = JMSLRandom.choose(0.2, 0.7);
//			double hold = duration * 0.8;
//			melody2.add(duration, pitch, amp, hold);
//		}

		// Prepare a Score, 2 staves, width, height
		Score score = new Score(2, 1024, 800);
		score.addMeasure();
		score.getMeasure(0).getStaff(1).setClef(Clef.BASS_CLEF);

		Transcriber transcriber = new Transcriber();
		transcriber.setScore(score);

		// Set up time signatures.  Only need to give it one 4/4 bar and transcriber will continue with 4/4 throughout
		TimeSignature ts = new TimeSignature(4, 4);
		Vector tsVector = new Vector();
		tsVector.addElement(ts);
		transcriber.setTimeSignatures(tsVector);

        transcriber.addTranscriberListener(new TranscriberListener() {

            public void noteAdded(Score score, Note note) {
                System.out.println("TranscribeScore12, TranscriberListener interface, adding note to score " + note.getPitchData());                
            }

            public void notifyCarriedOverMusicShape(Score score, int currentMeasureNumber, MusicShape musicShape) {
                // TODO Auto-generated method stub
                
            }});
		try {
			// transcribe first melody
			score.setCurrentStaffNumber(0);
			score.rewind();
			transcriber.setSourceMusicShape(melody1);
			melody1.integrate(0);
			transcriber.transcribe();

			// transcribe second melody
			score.setCurrentStaffNumber(1);
			score.rewind();
//			transcriber.setSourceMusicShape(melody2);
//			melody2.integrate(0);
//			transcriber.transcribe();

		}
		catch (ElementMissedException e) {
			e.printStackTrace();
			System.out.println("ERROR: " + e);
		}
		catch (SearchPathListExpansionException e) {
			e.printStackTrace();
			System.out.println("ERROR: " + e);
		}

		final ScoreFrame myScoreFrame = new ScoreFrame();
		myScoreFrame.addScore(score);
		//myScoreFrame.loadPrefs();
		myScoreFrame.addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent e) {
				myScoreFrame.quit();
			}
		});

		myScoreFrame.setVisible(true);
	}

}
