package jmsltestsuite.jsyn2;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import com.softsynth.jmsl.*;
import com.softsynth.jmsl.jsyn2.JSynMusicDevice;
import com.softsynth.jmsl.jsyn2.SamplePlayingInstrument;

public class TestSampleSustain {

	private MusicShape musicShape;
	private SamplePlayingInstrument instrument;
	private JMSLMixerContainer mixer;

	void buildMelody() {

		musicShape = new MusicShape(4);

		// for (int i = 0; i < instrument.POLYPHONY - 1; i++) {
		// musicShape.add(0.0, 48 + i, 0.5, 5.0);
		// }
		// musicShape.add(10.0, 40, 0.5, 5.0);

		// musicShape.add(2.0, 60, 0.5, 0.1);
		// musicShape.add(2.0, 60, 0.5, 0.2);
		// musicShape.add(2.0, 60, 0.5, 0.5);
		// musicShape.add(2.0, 60, 0.5, 1.0);
		// musicShape.add(3.0, 61, 0.5, 2.0);
		// musicShape.add(4.0, 60, 0.5, 3.0);
		// musicShape.add(5.0, 61, 0.5, 4.0);
		// musicShape.add(6.0, 60, 0.5, 5.0);
		double[] durs = { 0.25, 0.5, 1.0, 1.5, 2.0, 3.0, 4.0, 5.0 };
		for (int d = 0; d < durs.length; d++) {
			double dur = durs[d];
			for (int i = 0; i < 3; i++) {
				musicShape.add(dur, JMSLRandom.choose(58, 63), 0.8, dur * 0.8);
			}
		}
		musicShape.setInstrument(instrument);
	}

	void buildMixer() {
		mixer = new JMSLMixerContainer();
		mixer.addInstrument(instrument);
		mixer.start();
	}

	protected void play() {
		musicShape.setRepeats(Integer.MAX_VALUE);
		musicShape.launch(JMSL.now());
	}

	public void buildInstrument() {
		instrument = new SamplePlayingInstrument();
		instrument.setPlaybackMode(SamplePlayingInstrument.SAMPLE_CROSSFADED_LOOPING);
		instrument.setDirectory(new File("D:/documents/JMSLScoreWork/JMSLScoreSamples"));
		instrument.getMusicDevice().open();

		instrument.addSamplePitch("SIEDLACZEK/cello_bowed/c4.wav", 60);
		// instrument.addSamplePitch("yves/SopranoSaxShort.wav", 65);
		// instrument.addSamplePitch("yves/SopranoSaxOctave.wav", 77);

		instrument.buildFromAttributes();
	}

	public static void main(String args[]) {

		JMSL.clock.setAdvance(0.2);

		final TestSampleSustain test = new TestSampleSustain();
		test.buildInstrument();
		test.buildMixer();
		test.buildMelody();

		MusicJob jSynUsageJob = new MusicJob() {
			public double repeat(double playTime) throws InterruptedException {
				System.out.println("JSyn usage: " + JSynMusicDevice.instance().getSynthesizer().getUsage());
				return playTime + 0.2;
			}
		};

		jSynUsageJob.setRepeats(10000);
		// jSynUsageJob.launch(JMSL.now());

		java.awt.Frame f = new java.awt.Frame("JSyn2 TestSampleSustain");
		f.addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent e) {
				JMSL.closeMusicDevices();
				System.exit(0);
			}
		});
		f.add(test.mixer.getPanAmpControlPanel());
		Button stopButton;
		f.add(BorderLayout.SOUTH, stopButton = new Button("Start"));
		f.pack();
		f.setVisible(true);

		stopButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.out.println("Starting performance");
				test.play();
			}
		});
		f.pack();

	}

}
