/**
 * @author Nick Didkovsky, (c) 2000 Nick Didkovsky
 */

package jmsltutorial;

import java.awt.Color;
import java.awt.Label;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;

import com.softsynth.jmsl.EventScheduler;
import com.softsynth.jmsl.JMSL;
import com.softsynth.jmsl.JMSLRandom;
import com.softsynth.jmsl.jsyn2.JSynMusicDevice;
import com.softsynth.jmsl.jsyn2.JSynUnitVoiceInstrument;
import com.softsynth.jmsl.jsyn2.unitvoices.FMVoice;
import com.softsynth.jmsl.score.NoteFactory;
import com.softsynth.jmsl.score.Orchestra;
import com.softsynth.jmsl.score.Score;
import com.softsynth.jmsl.score.ScoreFrame;
import com.softsynth.jmsl.score.SelectionBuffer;

public class JScoreToot05 extends JFrame {

    Score score;
    ScoreFrame scoreFrame;

    public void init() {
        setBackground(Color.yellow);
        add(new Label("Your JMSL score will open in a new window"));
    }

    public void start() {
        JSynMusicDevice.instance().open();
        JMSL.clock.setAdvance(0.5);
        JMSL.scheduler = new EventScheduler();
        JMSL.scheduler.start();

        Orchestra orch = new Orchestra();
        orch.addInstrument(new JSynUnitVoiceInstrument(8, FMVoice.class.getName()), "Brass");
        orch.addInstrument(new JSynUnitVoiceInstrument(8, FMVoice.class.getName()), "Ring Bell");
        orch.buildMixer();
        orch.getJMSLMixerContainer().panAmpChange(0, 0, 0.5); // fader 0 hard left
        orch.getJMSLMixerContainer().panAmpChange(1, 1, 0.5); // fader 1 hard right
        // num staves, width, height
        score = new Score(2, 800, 600);
        score.addMeasure();
        score.setOrchestra(orch);

        scoreFrame = new ScoreFrame();

        scoreFrame.addScore(score);

        // Add wide range white noise melody, to demonstrate converge to
        // mean
        double dur = 0.25; // 16th note
        for (int i = 0; i < 32; i++) {
            score.addNote(dur, NoteFactory.MIDDLE_C + JMSLRandom.choose(36), 0.5, dur * 0.8);
            score.getLastAddedNote().setBeamedOut(i % 4 != 3);
        }

        // a few things to make the score easier to read:
        // 1) force these two measures to fit on one line
        double drawableWidth = score.getScoreLayoutManager().getDrawableWidth();
        score.getMeasure(0).setWidth(drawableWidth);
        score.getMeasure(1).setWidth(drawableWidth);
        // 2) lock width so measure does not calculate its own width
        score.getMeasure(0).setWidthSetByHand(true);
        score.getMeasure(1).setWidthSetByHand(true);
        // 3) clear some room above first staff since there will be high barred notes
        score.getMeasure(0).getStaff(0).setSpaceAbove(score.getMeasure(0).getStaff(0).getSpaceAbove() + 50);

        // Add your own transform to scoreFrame's menu
        scoreFrame.addUnaryCopyBufferTransform(new ConvergeToMeanTransform());
        scoreFrame.addUnaryCopyBufferTransform(new HocketTransform());
        scoreFrame.setSize(850, 500);
        scoreFrame.setVisible(true);
        score.render();
    }

    /* When applet stops, hide the scoreframe */
    public void stop() {
        if (scoreFrame != null) {
            scoreFrame.setVisible(false);
            scoreFrame.dispose();
            Score.deleteCanvas();
            SelectionBuffer.disposeEditFrame();
            scoreFrame = null;
            score = null;
        }
        JMSL.scheduler.stop();
        JMSL.closeMusicDevices();
    }

    public static void main(String args[]) {

        JScoreToot05 test = new JScoreToot05();
        test.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                test.stop();
                System.exit(0);
            }
        });

        test.init();
        test.start();
        test.pack();
        test.setVisible(true);
    }
}