package jmslexamples.jsyn2;

import java.awt.Graphics;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JApplet;

import com.jsyn.swing.JAppletFrame;
import com.softsynth.jmsl.EventScheduler;
import com.softsynth.jmsl.JMSL;
import com.softsynth.jmsl.jsyn2.JSynMusicDevice;
import com.softsynth.jmsl.jsyn2.JSynUnitVoiceInstrument;
import com.softsynth.jmsl.score.Clef;
import com.softsynth.jmsl.score.Orchestra;
import com.softsynth.jmsl.score.Score;
import com.softsynth.jmsl.score.ScoreFrame;

/**
 * Score with a JSyn instrument orchestra
 * 
 * @author Nick Didkovsky, copyright 2000 Nick Didkovsky
 */
public class JScoreJSynDemo {

    private Score score;
    private ScoreFrame scoreFrame;
    private Orchestra orch;

    public void start() {
        initJMSL();
        initMusicDevices();
        buildOrchestra();
        buildScore();
        buildScoreFrame();
    }

    private void initMusicDevices() {
        JSynMusicDevice.instance().open();
    }

    private void initJMSL() {       
        JMSL.clock.setAdvance(0.2);
        JMSL.scheduler = new EventScheduler();
        JMSL.scheduler.start();
    }

    private void termJMSL() {
        JMSL.scheduler.stop();
        JMSL.closeMusicDevices();
    }

    private void buildScoreFrame() {
        int w = score.getLayoutWidth();
        int h = score.getLayoutHeight();
        scoreFrame = new ScoreFrame("JMSL: Score & JSyn Demo by Nick Didkovsky, copyright 2000 Nick Didkovsky");
        scoreFrame.addScore(score);
        scoreFrame.setSize(w + 100, h + 100);
        scoreFrame.setVisible(true);
    }

    private void buildScore() {
        int w = 1000;
        int h = 700;
        int numStaves = 4;
        score = new Score(numStaves, w, h, "JMSL: Score & JSyn Demo by Nick Didkovsky, copyright 2000 Nick Didkovsky");
        score.addMeasure();
        // set bass clef for tuba
        score.getMeasure(0).getStaff(3).setClef(Clef.BASS_CLEF);
        score.setOrchestra(orch);
        score.getScoreLayoutManager().setZoom(0.5);
        score.getControlPanel().setZoom(0.5);

        score.setCurrentStaffNumber(0);
        score.addNote(1, 60, 0.5, 0.8);
        score.addNote(1, 60, 0.5, 0.8);
        score.addNote(1, 60, 0.5, 0.8);
        score.addNote(1, 60, 0.5, 0.8);
        score.rewind();
        score.setCurrentStaffNumber(1);
        score.addNote(1, 61, 0.5, 0.8);
        score.addNote(1, 61, 0.5, 0.8);
        score.addNote(1, 62, 0.5, 0.8);
        score.addNote(1, 62, 0.5, 0.8);

        score.rewind();
        score.setCurrentStaffNumber(2);
        score.addNote(2, 57, 0.5, 1.8);
        score.addNote(2, 62, 0.5, 1.8);

        score.rewind();
        score.setCurrentStaffNumber(3);
        score.addNote(4, 49, 0.5, 3.8);

    }

    private void buildOrchestra() {
        // Set up an JSynOrchestra to hand to Score, starts JSyn engine if it
        // needs to */
        orch = new Orchestra();
        orch.addInstrument(createBrassInstrument(), "trumpet 1");
        orch.addInstrument(createBrassInstrument(), "trumpet 2");
        orch.addInstrument(createBrassInstrument(), "horn");
        orch.addInstrument(createBrassInstrument(), "tuba");
    }

    private JSynUnitVoiceInstrument createBrassInstrument() {
        return new JSynUnitVoiceInstrument(1, com.softsynth.jmsl.jsyn2.unitvoices.FMVoice.class.getName(),
                com.softsynth.jmsl.jsyn2.unitvoices.FMVoice.PRESET_BRASS);
    }

    public ScoreFrame getScoreFrame() {
        return scoreFrame;
    }

    public void stop() {
        scoreFrame.setVisible(false);
        scoreFrame.dispose();
        Score.deleteCanvas();
        termJMSL();
    }
    
    public static void main(String args[]) {
        JScoreJSynDemo demo = new JScoreJSynDemo();
        demo.start();
        demo.getScoreFrame().addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                JMSL.closeMusicDevices();
                System.exit(0);
            }
        });
    }

}