package jmslexamples.jsyn2;

import java.awt.Color;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JFrame;

import com.didkovsky.portview.PVButton;
import com.didkovsky.portview.PVCheckbox;
import com.didkovsky.portview.PVPanel;
import com.didkovsky.portview.swing.ViewFactorySwing;
import com.softsynth.jmsl.JMSL;
import com.softsynth.jmsl.JMSLRandom;
import com.softsynth.jmsl.MusicDevice;
import com.softsynth.jmsl.MusicJob;
import com.softsynth.jmsl.MusicShape;
import com.softsynth.jmsl.PlayLurker;
import com.softsynth.jmsl.jsyn2.JSynMusicDevice;
import com.softsynth.jmsl.jsyn2.JSynUnitVoiceInstrument;
import com.softsynth.jmsl.jsyn2.unitvoices.FilteredSawtoothBL;
import com.softsynth.jmsl.score.Measure;
import com.softsynth.jmsl.score.NoteFlasher;
import com.softsynth.jmsl.score.Orchestra;
import com.softsynth.jmsl.score.Score;
import com.softsynth.jmsl.score.ScoreCollectionBehavior;
import com.softsynth.jmsl.score.Tempo;
import com.softsynth.jmsl.score.transcribe.BeatDivisionSchemeList;
import com.softsynth.jmsl.score.transcribe.ElementMissedException;
import com.softsynth.jmsl.score.transcribe.SearchPathListExpansionException;
import com.softsynth.jmsl.score.transcribe.TempoTimeSignatureHolder;
import com.softsynth.jmsl.score.transcribe.Transcriber;
import com.softsynth.jmsl.score.util.JMSLScoreApplet;
import com.softsynth.jmsl.util.Oof;
import com.softsynth.jmsl.util.TimeSignature;
import com.softsynth.jmsl.view.PVPanelAdapter;

/**
 * Launch more than one score simultaneously so you can do multi-tempo pieces,
 * etc
 * 
 * @author Nick Didkovsky, Aug 21, 2011, (c) 2011 Nick Didkovsky, all rights
 *         reserved
 */

public class Nancarrow extends JFrame implements ActionListener {

    PVPanel panel;
    PVButton goButton;
    PVButton finishButton;
    PVCheckbox loopCheckbox;
    Vector scores = new Vector();

    public Nancarrow() {
        panel = new PVPanelAdapter();
        goButton = JMSL.getViewFactory().createButton("LAUNCH SCORES IN PARALLEL");
        panel.add(goButton.getComponent());
        finishButton = JMSL.getViewFactory().createButton("FINISH");
        panel.add(finishButton.getComponent());
        panel.add((loopCheckbox = JMSL.getViewFactory().createCheckbox("Loop", false)).getComponent());
        goButton.addActionListener(this);
        finishButton.addActionListener(this);
    }

    public void addScore(Score score) {
        scores.addElement(score);
    }

    public PVPanel getPanel() {
        return panel;
    }

    public static Score generateSomeScore(double tempo, double octave) {
        Score score = new Score(1, 1024, 200, "Study in " + tempo + " bpm");
        score.ownCanvas(); // !!!

        Orchestra orch = new Orchestra();
        orch.addInstrument(new JSynUnitVoiceInstrument(4, FilteredSawtoothBL.class.getName()));
        score.setOrchestra(orch);

        double[] durations = { 0.5, 0.25, 1.0, 0.333 };
        Measure measure = score.addMeasure();
        MusicShape musicShape = new MusicShape(4);
        double density = JMSLRandom.choose(0.25, 4.0);
        double ratio = JMSLRandom.choose(1.01, 12.0);
        double totalDur = 0;
        double maxDur = 20;
        Oof oof = new Oof(8);
        oof.randomize();
        while (totalDur < maxDur) {
            // double duration = EventDistributions.genEntryDelayMyhill(density,
            // ratio);
            double duration = durations[JMSLRandom.choose(durations.length)];
            duration = 60 / tempo * duration;
            double pitch = oof.next();
            pitch = constrainPitch(pitch, octave);
            double amp = JMSLRandom.choose(0.2, 0.8);
            musicShape.add(totalDur, pitch, amp, duration * 0.8);
            totalDur += duration;
        }
        Vector timeSignatures = new Vector();
        int[] demons = { 4, 8, 16, 4 };
        for (int i = 0; i < 5; i++) {
            TimeSignature timeSignature = new TimeSignature(JMSLRandom.choose(2, 8),
                    demons[JMSLRandom.choose(demons.length)]);
            timeSignatures.addElement(new TempoTimeSignatureHolder(new Tempo(tempo), timeSignature));
        }
        Transcriber transcriber = new Transcriber();
        BeatDivisionSchemeList.defaultSetup();
        transcriber.setScore(score);
        transcriber.setTimeSignatures(timeSignatures);
        transcriber.setSourceMusicShape(musicShape);
        try {
            transcriber.transcribe();
        } catch (SearchPathListExpansionException e) {
            e.printStackTrace();
        } catch (ElementMissedException e) {
            e.printStackTrace();
        }

        score.getScoreCollection().addPlayLurker(new PlayLurker() {
            public void notifyPlayLurker(double playTime, MusicJob list, int index) {
                System.out.println(list.getName() + " playing measure " + index);
            }
        });
        if (JMSLRandom.choose() < 0.8) {
            int measureindex = JMSLRandom.choose(score.size());
            Measure measureToBeRepeated = score.getMeasure(measureindex);
            measureToBeRepeated.setRepeatStart(true);
            measureToBeRepeated.setRepeatEnd(true);
            measureToBeRepeated.setNumRepeats(JMSLRandom.choose(2, 4));
        }
        return score;
    }

    private static double constrainPitch(double pitch, double octave) {
        double bassPitch = 48 + octave * 12;
        while (pitch < bassPitch)
            pitch += 12;
        while (pitch > bassPitch + 14)
            pitch -= 12;
        return pitch;
    }

    public void actionPerformed(ActionEvent ev) {
        Object source = ev.getSource();
        if (source == goButton) {
            double when = JMSL.now();
            for (Enumeration e = scores.elements(); e.hasMoreElements();) {
                Score score = (Score) e.nextElement();
                score.getControlPanel().setLoop(loopCheckbox.getState());                
//                score.getControlPanel().setNoteFlash(false);               
                score.getControlPanel().launch(when, 0, score.size() - 1, false);
            }
        }
        if (source == finishButton) {
            for (Enumeration e = scores.elements(); e.hasMoreElements();) {
                Score score = (Score) e.nextElement();
                score.getControlPanel().finish();
                ScoreCollectionBehavior behavior = (ScoreCollectionBehavior) score.getScoreCollection().getBehavior();
                ;
                int measureWhereFinished = behavior.getCurrentMeasureNumber();
                System.out.println("Score " + score.getName() + " finished on measure " + measureWhereFinished);
            }
        }
    }

    public void start() {

        // CRITICAL!!! Each new Score must own its own ScoreCanvas for rendering
        Score.useSharedCanvas(false);

        JMSLRandom.randomize();
        JMSL.setViewFactory(new ViewFactorySwing());
        
        JMSL.clock.setAdvance(0.3);
        NoteFlasher.setFlashLatency(0.3);
        
        // f= new
        // PVFrameAdapter("Different scores in different tempi, play in parallel!");
        // f.setSize(1100, 600);
        setLayout(new GridLayout(0, 1));

        initMusicDevice();

        PVPanel p = new PVPanelAdapter();
        p.setLayout(new GridLayout(0, 3, 10, 10));

        Nancarrow multiScoreLauncher = new Nancarrow();

        // create a new score for each tempo
        double[] tempi = { 60, 72, 50 };
        double[] pans = { 0.0, 0.5, 1.0 };

        for (int i = 0; i < tempi.length; i++) {
            double octave = i;
            Score score = Nancarrow.generateSomeScore(tempi[i], octave);
            multiScoreLauncher.addScore(score);
            add(score.getScoreCanvas().getComponent());
            double pan = pans[i];
            score.getOrchestra().getJMSLMixerContainer().panAmpChange(0, pan, 0.5);
            // f2.add(score.getOrchestra().getJMSLMixerContainer().getPanAmpControlPanel(1,
            // 1, 0, 0));
            score.getScorePainter().setBGColor(randomBrightColor());
        }
        p.add(multiScoreLauncher.getPanel().getComponent());
        add(p.getComponent());
        // ScoreCommandCenter.addScoreCommand(new
        // NotePropertiesTransformCommand(new RestToggleTransform()),
        // KeyEvent.VK_R);
    }

    private Color randomBrightColor() {
        return new Color(JMSLRandom.choose(220, 256), JMSLRandom.choose(220, 256), JMSLRandom.choose(240, 256));
    }

    private void initMusicDevice() {
        MusicDevice dev = JSynMusicDevice.instance();
        dev.edit(new Frame());
        dev.open();
    }

    public static void main(String[] args) {
        Nancarrow demo = new Nancarrow();
        demo.start();
        demo.pack();
        demo.setVisible(true);
        demo.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                JMSL.closeMusicDevices();
                System.exit(0);
            }
        });

    }

}